## FILE GENERATED BY mProv.  Modifications will be overwritten
# Please modify the mProv template.

# host-record=<name>[,<name>....],[<IPv4-address>],[<IPv6-address>][,<TTL>]
{% for host in hosts %}
{{host.ipaddress}} {{host.hostname}} {{host.hostname}}.{{host.domain}}
{% endfor %}

