## FILE GENERATED BY mProv.  Modifications will be overwritten
# Please modify the mProv template.

# dhcp-host=[<hwaddr>][,id:<client_id>|*][,set:<tag>][tag:<tag>][,<ipaddr>][,<hostname>][,<lease_time>][,ignore]
{% for host in hosts %}
{{host.mac}},set:{{host.network}},{{host.ipaddress}},{{host.hostname}}
{% endfor %}