## FILE GENERATED BY mProv.  Modifications will be overwritten
# Please modify the mProv template.
{%if enableDNS %}
# do not read /etc/hosts.
no-hosts

# do read in the mprov hosts files in /var/lib/mprov
hostsdir=/var/lib/mprov/dns/

# only reply to interfaces we are listening on
local-service

# do not forward short names upstream
domain-needed

# add in our domains here
{% for network in networks %}
auth-zone={{network.domain}},{{network.subnet}}/{{network.netmask}}
{% endfor %}
{% endif %}
